<?php

namespace Drupal\BasicShib\Logger;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LogMessageParserInterface;
use Drupal\Core\Logger\RfcLoggerTrait;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\Core\Messenger\MessengerInterface;
use Psr\Log\LoggerInterface;

class ShibLogger implements LoggerInterface {

  use RfcLoggerTrait;

  /**
   * The log message parser.
   *
   * @var \Drupal\Core\Logger\LogMessageParserInterface
   */
  protected LogMessageParserInterface $parser;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * Include the messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected MessengerInterface $messenger;

  /**
   * Include logging options.
   *
   * @var array
   */
  protected array $logging;

  /**
   * ShibLogger constructor.
   *
   * @param \Drupal\Core\Logger\LogMessageParserInterface $parser
   *   The message parser.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   */
  public function __construct(LogMessageParserInterface $parser, ConfigFactoryInterface $config_factory, MessengerInterface $messenger) {
    $this->parser = $parser;
    $this->configFactory = $config_factory;
    $this->messenger = $messenger;
    $this->logging = $config_factory->get('basicshib.settings')->get('extended_logging');
  }

  /**
   * If set on the BasicShib Configuration form the watchdog messages are also
   * displayed as a message, useful in debugging authentication and authorization
   * problems.
   *
   * @param $level
   * @param $message
   * @param array $context
   *
   * @return void
   */
  public function log($level, $message, array $context = []) {

   // If extended logging messaging set output log entry to message area
   if ($this->logging['messaging'] == TRUE) {
      $levels = RfcLogLevel::getLevels();

      switch ($level) {
        case RfcLogLevel::EMERGENCY:
        case RfcLogLevel::ALERT:
        case RfcLogLevel::CRITICAL:
        case RfcLogLevel::ERROR:
          $this->messenger->addError($levels[$level] . ': ' . $message);
          break;

        case RfcLogLevel::WARNING:
        case RfcLogLevel::NOTICE:
          $this->messenger->addWarning($levels[$level] . ': ' . $message);
          break;

        case RfcLogLevel::INFO:
          $this->messenger->addStatus($levels[$level] . ': ' . $message);
          break;

        case RfcLogLevel::DEBUG:
          $this->messenger->addMessage($levels[$level] . ': ' . $message);
          break;
      }
    }
  }
}
